@@ -1,32 +1,88 @@
 package Text::Template::Simple::Base::Parser;
 use strict;
 use vars qw($VERSION);
+use Carp qw( croak );
+use Text::Template::Simple::Util;
+use Text::Template::Simple::Constants;
 
-$VERSION = '0.79_09';
+$VERSION = '0.60';
 
-use Text::Template::Simple::Util      qw(:all);
-use Text::Template::Simple::Constants qw(:all);
+# internal code templates
+my %INTERNAL = (
+   # we need string eval in this template to catch syntax errors
+   sub_include => q~
+      <%OBJECT%>->_compile(
+         do {
+            local $@;
+            my $file = eval '<%INCLUDE%>';
+            my $rv;
+            if ( my $e = $@ ) {
+               chomp $e;
+               $file ||= '<%INCLUDE%>';
+               my $m = "The parameter ($file) is not a file. "
+                     . "Error from sub-include ($file): $e";
+               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
+            }
+            else {
+               $rv = $file;
+            }
+            $rv;
+         },
+         undef,
+         {
+            _sub_inc => '<%TYPE%>'
+         }
+      )
+   ~,
+   no_monolith => q*
+      <%OBJECT%>->compile(
+         q~<%FILE%>~,
+         undef,
+         {
+            chkmt    => 1,
+            _sub_inc => q~<%TYPE%>~,
+         }
+      );
+   *,
 
-my %INTERNAL = __PACKAGE__->_set_internal_templates;
+   # see _parse()
+   map_keys_check => q(
+      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
+               ? (
+                  defined <%HASH%>->{"<%KEY%>"}
+                  ? <%HASH%>->{"<%KEY%>"}
+                  : "[ERROR] Key not defined: <%KEY%>"
+                  )
+               : "[ERROR] Invalid key: <%KEY%>"
+               ;
+   ),
+
+   map_keys_init => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
+   ),
+   map_keys_default => q(
+      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
+   ),
+);
 
 sub _internal {
    my $self = shift;
-   my $id   = shift            || fatal('tts.base.parser._internal.id');
-   my $rv   = $INTERNAL{ $id } || fatal('tts.base.parser._internal.id');
-   LOG( INTERNAL => "TEMPLATE: $id" ) if DEBUG;
+   my $id   = shift            || croak "_internal(): id is missing";
+   my $rv   = $INTERNAL{ $id } || croak "_internal(): id is invalid";
    return $rv;
 }
 
 sub _parse {
-   my($self, $raw, $opt) = @_;
-
-   # $opt->
-   #      map_keys: code sections are hash keys
-   #      as_is   : i.e.: do not parse -> static include
-
+   my $self     = shift;
+   my $raw      = shift;
+   my $map_keys = shift; # code sections are hash keys
+   my $cache_id = shift;
+   my $as_is    = shift; # i.e.: do not parse -> static include
    #$self->[NEEDS_OBJECT] = 0; # reset
 
-   my($ds, $de) = @{ $self->[DELIMITERS] };
+   my $resume   = $self->[RESUME] || '';
+   my $ds       = $self->[DELIMITERS][DELIM_START];
+   my $de       = $self->[DELIMITERS][DELIM_END  ];
    my $faker    = $self->[INSIDE_INCLUDE] ? $self->_output_buffer_var
                                           : $self->[FAKER]
                                           ;
@@ -37,91 +93,76 @@ sub _parse {
    my $code     = '';
    my $inside   = 0;
 
-   my($mko, $mkc) = $self->_parse_mapkeys( $opt->{map_keys}, $faker, $buf_hash );
+   my($mko, $mkc) = $self->_parse_mapkeys( $map_keys, $faker, $buf_hash );
 
    LOG( RAW => $raw ) if ( DEBUG() > 3 );
 
-   my $uth = $self->[USER_THANDLER];
+   my $handler = $self->[USER_THANDLER];
 
-   my $h = {
-      raw     => sub { ";$faker .= q~$_[0]~;" },
-      capture => sub { ";$faker .= sub {" . $_[0] . "}->();"; },
-      code    => sub { $_[0] . ';' },
-   };
+   my $w_raw = sub { ";$faker .= q~$_[0]~;" };
+   my $w_cap = sub { ";$faker .= sub {" . $_[0] . "}->();"; };
 
    # little hack to convert delims into escaped delims for static inclusion
-   $raw =~ s{\Q$ds}{$ds!}xmsg if $opt->{as_is};
+   $raw =~ s{\Q$ds}{$ds!}xmsg if $as_is;
 
    # fetch and walk the tree
-   PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $opt->{map_keys} ) } ) {
+   PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $map_keys ) } ) {
       my($str, $id, $chomp, undef) = @{ $token };
-      LOG( TOKEN => $toke->_visualize_tid($id) . " => $str" ) if DEBUG() > 1;
-      next PARSER if T_DISCARD == $id || T_COMMENT == $id;
+      LOG( TOKEN => "$id => $str" ) if DEBUG() > 1;
+      next PARSER if $id eq 'DISCARD';
+      next PARSER if $id eq 'COMMENT';
 
-      if ( T_DELIMSTART == $id ) { $inside++; next PARSER; }
-      if ( T_DELIMEND   == $id ) { $inside--; next PARSER; }
+      if ( $id eq 'DELIMSTART' ) { $inside++; next PARSER; }
+      if ( $id eq 'DELIMEND'   ) { $inside--; next PARSER; }
 
-      if ( T_RAW == $id || T_NOTADELIM == $id ) {
-         $code .= $h->{raw}->( $self->_chomp( $str, $chomp ) );
+      if ( $id eq 'RAW' || $id eq 'NOTADELIM' ) {
+         $code .= $w_raw->( $self->_chomp( $str, $chomp ) );
       }
 
-      elsif ( T_CODE == $id ) {
-         $code .= $h->{code}->($str);
+      elsif ( $id eq 'CODE' ) {
+         $code .= $resume ? $self->_resume($str, 0, 1) : $str;
       }
 
-      elsif ( T_CAPTURE == $id ) {
-         $code .= $h->{capture}->( $str );
+      elsif ( $id eq 'CAPTURE' ) {
+         $code .= $faker;
+         $code .= $resume ? $self->_resume($str, RESUME_NOSTART)
+                :           " .= sub { $str }->();";
       }
 
-      elsif ( T_DYNAMIC == $id || T_STATIC == $id ) {
-         $code .= $h->{capture}->( $self->_needs_object->_include($id, $str, $opt) );
+      elsif ( $id eq 'DYNAMIC' || $id eq 'STATIC' ) {
+         $self->[NEEDS_OBJECT]++;
+         $code .= $w_cap->( $self->_include($id, $str) );
       }
 
-      elsif ( T_MAPKEY == $id ) {
+      elsif ( $id eq 'MAPKEY' ) {
          $code .= sprintf $mko, $mkc ? ( ($str) x 5 ) : $str;
       }
 
-      elsif ( T_COMMAND == $id ) {
-         $code .= $h->{raw}->( $self->_parse_command( $str ) );
-      }
-
       else {
-         LOG(
-            $uth  ? (USER_THANDLER => "$id")
-                  : (UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
-         ) if DEBUG;
-         $code .= $uth ? $uth->( $self, $id ,$str, $h ) : $h->{raw}->( $str );
+         if ( $handler ) {
+            LOG( USER_THANDLER => "$id") if DEBUG;
+            $code .= $handler->(
+                        $self, $id ,$str, { capture => $w_cap, raw => $w_raw }
+                     );
+         }
+         else {
+            LOG( UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
+               if DEBUG;
+            $code .= $w_raw->($str);
+         }
       }
 
    }
 
    $self->[FILENAME] ||= '<ANON>';
 
-   fatal(
-      'tts.base.parser._parse.unbalanced',
-         abs($inside),
-         ($inside > 0 ? 'opening' : 'closing'),
-         $self->[FILENAME]
-   ) if $inside;
-
-   return $self->_wrapper( $code, $opt->{cache_id}, $faker, $opt->{map_keys}, $h );
-}
-
-sub _parse_command {
-   my $self = shift;
-   my $str  = shift;
-   my($head, $raw_block) = split /;/, $str, 2;
-   my @buf  = split RE_PIPE_SPLIT, '|' . trim($head);
-   shift(@buf);
-   my %com  = map { trim $_ } @buf;
-
-   if ( $com{FILTER} ) {
-      # embed into the template & NEEDS_OBJECT++ ???
-      local $self->[FILENAME] = '<ANON BLOCK>';
-      $self->_call_filters( \$raw_block, split RE_FILTER_SPLIT, $com{FILTER} );
+   if ( $inside ) {
+      my $type = $inside > 0 ? 'opening' : 'closing';
+      my $tmpl = "%d unbalanced %s delimiter(s) in template %s";
+      croak sprintf( $tmpl, abs($inside), $type, $self->[FILENAME] );
    }
 
-   return $raw_block;
+   return $self->_wrapper( $code, $cache_id, $faker, $map_keys );
 }
 
 sub _chomp {
@@ -161,7 +202,6 @@ sub _wrapper {
    my $cache_id = shift;
    my $faker    = shift;
    my $map_keys = shift;
-   my $h        = shift;
    my $buf_hash = $self->[FAKER_HASH];
 
    my $wrapper    = '';
@@ -183,23 +223,17 @@ sub _wrapper {
    $wrapper .= "my $faker = '';";
    $wrapper .= $self->_add_stack( $cache_id )  if $self->[STACK];
    $wrapper .= "my $buf_hash = {\@_};"         if $map_keys;
-   $wrapper .= $self->_add_sigwarn if $self->[CAPTURE_WARNINGS];
    $wrapper .= "\n#line 1 " .  $self->[FILENAME] . "\n";
-   $wrapper .= $code . ";";
-   $wrapper .= $self->_dump_sigwarn($h) if $self->[CAPTURE_WARNINGS];
-   $wrapper .= "return $faker;";
+   $wrapper .= $code . ";return $faker;";
    $wrapper .= '}';
    # make this a capture sub if we're including
    $wrapper .= '->()' if $inside_inc;
 
-   LOG( COMPILED =>  $self->_mini_compiler(
-                        $self->_internal('fragment'),
-                        { FRAGMENT => $self->_tidy($wrapper) }
-                     )
-   ) if DEBUG > 1;
+   LOG( COMPILED => sprintf FRAGMENT_TMP, $self->_tidy($wrapper) )
+      if DEBUG() > 1;
    #LOG( OUTPUT => $wrapper );
    # reset
-   $self->[DEEP_RECURSION] = 0; # reset
+   $self->[DEEP_RECURSION] = 0 if $self->[DEEP_RECURSION];
    return $wrapper;
 }
 
@@ -225,30 +259,6 @@ sub _parse_mapkeys {
    return $mko, $mkc;
 }
 
-sub _add_sigwarn {
-   my $self = shift;
-   $self->[FAKER_WARN] = $self->_output_buffer_var('array');
-   my $rv = $self->_mini_compiler(
-               $self->_internal('add_sigwarn'),
-               { BUF     => $self->[FAKER_WARN] },
-               { flatten => 1                   }
-            );
-   return $rv;
-}
-
-sub _dump_sigwarn {
-   my $self = shift;
-   my $h    = shift;
-   my $rv = $h->{capture}->(
-               $self->_mini_compiler(
-                  $self->_internal('dump_sigwarn'),
-                  { BUF     => $self->[FAKER_WARN] },
-                  { flatten => 1                   }
-               )
-            );
-   return $rv;
-}
-
 sub _add_stack {
    my $self    = shift;
    my $cs_name = shift || '<ANON TEMPLATE>';
@@ -274,116 +284,41 @@ sub _add_stack {
    return "$channel stack( { type => '$type', name => '$cs_name' } );";
 }
 
-sub _set_internal_templates {
-   # we need string eval in this template to catch syntax errors
-   sub_include => q~
-      <%OBJECT%>->_compile(
-         do {
-            local $@;
-            my $file = eval '<%INCLUDE%>';
-            my $rv;
-            if ( my $e = $@ ) {
-               chomp $e;
-               $file ||= '<%INCLUDE%>';
-               my $m = "The parameter ($file) is not a file. "
-                     . "Error from sub-include ($file): $e";
-               $rv = [ ERROR => '<%ERROR_TITLE%> ' . $m ]
-            }
-            else {
-               $rv = $file;
-            }
-            $rv;
-         },
-         <%PARAMS%>,
-         {
-            _sub_inc => '<%TYPE%>',
-            _filter  => '<%FILTER%>',
-            _share   => [<%SHARE%>],
+# TODO: unstable. consider removing this thing (also the constants)
+sub _resume {
+   my $self    = shift;
+   my $token   = shift           || return;
+   my $nostart = shift           || 0;
+   my $is_code = shift           || 0;
+   my $resume  = $self->[RESUME] || '';
+   my $start   = $nostart ? '' : $self->[FAKER];
+   my $void    = $nostart ? 0  : 1; # not a self-printing block
+
+   if ( $token && $resume && $token !~ RESUME_MY ) {
+      if (
+            $token !~ RESUME_CURLIES &&
+            $token !~ RESUME_ELSIF   &&
+            $token !~ RESUME_ELSE    &&
+            $token !~ RESUME_LOOP
+      ) {
+         LOG( RESUME_OK => $token ) if DEBUG() > 2;
+         my $rvar        = $self->_output_buffer_var('array');
+         my $resume_code = RESUME_TEMPLATE;
+         foreach my $replace (
+            [ RVAR  => $rvar             ],
+            [ TOKEN => $token            ],
+            [ PID   => $self->_class_id  ],
+            [ VOID  => $void             ],
+         ) {
+            $resume_code =~ s{ <% $replace->[0] %> }{$replace->[1]}xmsg;
          }
-      )
-   ~,
+         return $start . $resume_code;
+      }
+   }
 
-   no_monolith => q*
-      <%OBJECT%>->compile(
-         q~<%FILE%>~,
-         undef,
-         {
-            chkmt    => 1,
-            _sub_inc => q~<%TYPE%>~,
-         }
-      );
-   *,
+   LOG( RESUME_NOT => $token ) if DEBUG() > 2;
 
-   # see _parse()
-   map_keys_check => q(
-      <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
-               ? (
-                  defined <%HASH%>->{"<%KEY%>"}
-                  ? <%HASH%>->{"<%KEY%>"}
-                  : "[ERROR] Key not defined: <%KEY%>"
-                  )
-               : "[ERROR] Invalid key: <%KEY%>"
-               ;
-   ),
-
-   map_keys_init => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';
-   ),
-
-   map_keys_default => q(
-      <%BUF%> .= <%HASH%>->{"<%KEY%>"};
-   ),
-
-   add_sigwarn => q(
-      my <%BUF%>;
-      local $SIG{__WARN__} = sub {
-         push @{ <%BUF%> }, $_[0];
-      };
-   ),
-
-   dump_sigwarn => q(
-      join("\n",
-            map {
-               s{ \A \s+    }{}xms;
-               s{    \s+ \z }{}xms;
-               "[warning] $_\n"
-            } @{ <%BUF%> }
-         );
-   ),
-
-   compile_error => <<'TEMPLATE_CONSTANT',
-Error compiling code fragment (cache id: <%CID%>):
-
-<%ERROR%>
--------------------------------
-PARSED CODE (VERBATIM):
--------------------------------
-
-<%PARSED%>
-
--------------------------------
-PARSED CODE    (tidied):
--------------------------------
-
-<%TIDIED%>
-TEMPLATE_CONSTANT
-
-   fragment => <<'TEMPLATE_CONSTANT',
-
-# BEGIN TIDIED FRAGMENT
-
-<%FRAGMENT%>
-
-# END TIDIED FRAGMENT
-TEMPLATE_CONSTANT
-
-   disk_cache_comment => <<"TEMPLATE_CONSTANT",
-# !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
-# This file was automatically generated by <%NAME%> on <%DATE%>.
-# This file is a compiled template cache.
-# Any changes you make here will be lost.
-#
-TEMPLATE_CONSTANT
+   return $is_code ? $token : "$start .= $token;"
 }
 
 1;
@@ -400,13 +335,6 @@ Private module.
 
 =head1 DESCRIPTION
 
-This document describes version C<0.79_09> of C<Text::Template::Simple::Base::Parser>
-released on C<7 August 2009>.
-
-B<WARNING>: This version of the module is part of a
-developer (beta) release of the distribution and it is
-not suitable for production use.
-
 Private module.
 
 =begin CHOMPING
@@ -428,23 +356,22 @@ If the previous or next is not raw, nothing will happen. You need to swap sides
 when handling the chomping. i.e.: left chomping affects the right side of the
 RAW, and right chomping affects the left side of the RAW. _chomp() method in
 the parser swaps sides to handle chomping.
-See Text::Template::Simple::Tokenizer to have an idea on how pre-parsing
-happens.
+See Text::Template::Simple::Tokenizer to see how pre-parsing happens.
 
 =end CHOMPING
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004 - 2009 Burak Gursoy. All rights reserved.
+Copyright 2004-2008 Burak GE<252>rsoy. All rights reserved.
 
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.10.0 or, 
+it under the same terms as Perl itself, either Perl version 5.8.8 or, 
 at your option, any later version of Perl 5 you may have available.
 
 =cut
