@@ -2,61 +2,95 @@ package Text::Template::Simple::Constants;
 use strict;
 use vars qw($VERSION $OID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.53';
+$VERSION = '0.60';
 
 # object fields
 BEGIN { $OID = -1 } # init object field id counter
-use constant DELIMITERS     => ++$OID;
-use constant AS_STRING      => ++$OID;
-use constant DELETE_WS      => ++$OID;
-use constant FAKER          => ++$OID;
-use constant FAKER_HASH     => ++$OID;
-use constant CACHE          => ++$OID;
-use constant CACHE_DIR      => ++$OID;
-use constant CACHE_OBJECT   => ++$OID;
-use constant IO_OBJECT      => ++$OID;
-use constant STRICT         => ++$OID;
-use constant SAFE           => ++$OID;
-use constant HEADER         => ++$OID;
-use constant ADD_ARGS       => ++$OID;
-use constant WARN_IDS       => ++$OID;
-use constant FIX_UNCUDDLED  => ++$OID;
-use constant TYPE           => ++$OID;
-use constant COUNTER        => ++$OID;
-use constant CID            => ++$OID;
-use constant FILENAME       => ++$OID;
-use constant RESUME         => ++$OID;
-use constant IOLAYER        => ++$OID;
-use constant STACK          => ++$OID;
-use constant USER_THANDLER  => ++$OID;
-# number of the last object field
-use constant MAXOBJFIELD    =>   $OID;
+use constant DELIMITERS       => ++$OID;
+use constant AS_STRING        => ++$OID;
+use constant DELETE_WS        => ++$OID;
+use constant FAKER            => ++$OID;
+use constant FAKER_HASH       => ++$OID;
+use constant FAKER_SELF       => ++$OID;
+use constant MONOLITH         => ++$OID;
+use constant CACHE            => ++$OID;
+use constant CACHE_DIR        => ++$OID;
+use constant CACHE_OBJECT     => ++$OID;
+use constant IO_OBJECT        => ++$OID;
+use constant STRICT           => ++$OID;
+use constant SAFE             => ++$OID;
+use constant HEADER           => ++$OID;
+use constant ADD_ARGS         => ++$OID;
+use constant WARN_IDS         => ++$OID;
+use constant TYPE             => ++$OID;
+use constant TYPE_FILE        => ++$OID;
+use constant COUNTER          => ++$OID;
+use constant COUNTER_INCLUDE  => ++$OID;
+use constant INSIDE_INCLUDE   => ++$OID;
+use constant NEEDS_OBJECT     => ++$OID;
+use constant CID              => ++$OID;
+use constant FILENAME         => ++$OID;
+use constant RESUME           => ++$OID;
+use constant IOLAYER          => ++$OID;
+use constant STACK            => ++$OID;
+use constant USER_THANDLER    => ++$OID;
+use constant DEEP_RECURSION   => ++$OID;
+use constant INCLUDE_PATHS    => ++$OID;
+use constant PRE_CHOMP        => ++$OID;
+use constant POST_CHOMP       => ++$OID;
+use constant MAXOBJFIELD      =>   $OID; # number of the last object field
 
-use constant PARENT         => 'Text::Template::Simple';
-use constant IS_WINDOWS     => $^O eq 'MSWin32' || $^O eq 'MSWin64';
-use constant DELIM_START    => 0;
-use constant DELIM_END      => 1;
-use constant RE_NONFILE     => qr{ [ \n \r < > \* \? ] }xmso;
-use constant RE_DUMP_ERROR  => qr{Can\'t locate object method "first" via package "B::SVOP"};
-use constant RESUME_NOSTART => 1;                         # bool
-use constant COMPILER       => PARENT.'::Compiler';       # The compiler
-use constant COMPILER_SAFE  => PARENT.'::Compiler::Safe'; # Safe compiler
-use constant DUMMY_CLASS    => PARENT.'::Dummy';          # Dummy class
-use constant MAX_FL         => 80;                        # Maximum file name length
-use constant CACHE_EXT      => '.tmpl.cache';             # disk cache extension
-use constant STAT_SIZE      => 7;                         # for stat()
-use constant STAT_MTIME     => 9;                         # for stat()
-use constant DELIMS         => qw( <% %> );               # default delimiter pair
-use constant NEW_PERL       => $] >= 5.008;               # for I/O layer
+# settings
+use constant MAX_RECURSION    => 50; # recursion limit for dynamic includes
+use constant PARENT           => 'Text::Template::Simple';
+use constant IS_WINDOWS       => $^O eq 'MSWin32' || $^O eq 'MSWin64';
+use constant DELIM_START      => 0; # field id
+use constant DELIM_END        => 1; # field id
+use constant RE_NONFILE       => qr{ [ \n \r < > \* \? ] }xmso;
+use constant RE_DUMP_ERROR    => qr{Can\'t locate object method "first" via package "B::SVOP"};
+use constant RESUME_NOSTART   => 1;                   # bool
+use constant COMPILER         => PARENT.'::Compiler'; # The compiler
+use constant COMPILER_SAFE    => COMPILER.'::Safe';   # Safe compiler
+use constant DUMMY_CLASS      => PARENT.'::Dummy';    # Dummy class
+use constant MAX_FL           => 120;                 # Maximum file name length
+use constant CACHE_EXT        => '.tts.cache';        # disk cache extension
+use constant STAT_SIZE        => 7;                   # for stat()
+use constant STAT_MTIME       => 9;                   # for stat()
+use constant DELIMS           => qw( <% %> );         # default delimiter pair
+use constant NEW_PERL         => $] >= 5.008;         # for I/O layer
+use constant IS_FLOCK         => IS_WINDOWS ? ( Win32::IsWin95() ? 0 : 1 ) : 1;
 
-use constant IS_FLOCK       => sub {
-   # are we running under dumb OS?
-   if ( IS_WINDOWS ) {
-      return Win32::IsWin95() ? 0 : 1;
-   }
-   return 1; # TODO: test flock() directly at this point
-   return 0; # disable by default
-}->();
+use constant CHOMP_NONE       => 0x000000;
+use constant COLLAPSE_NONE    => 0x000000;
+use constant CHOMP_ALL        => 0x000002;
+use constant CHOMP_LEFT       => 0x000004;
+use constant CHOMP_RIGHT      => 0x000008;
+use constant COLLAPSE_LEFT    => 0x000010;
+use constant COLLAPSE_RIGHT   => 0x000020;
+use constant COLLAPSE_ALL     => 0x000040;
+
+# first level directives
+use constant DIR_CAPTURE      => '=';
+use constant DIR_DYNAMIC      => '*';
+use constant DIR_STATIC       => '+';
+use constant DIR_NOTADELIM    => '!';
+use constant DIR_COMMENT      => '#';
+# second level directives
+use constant DIR_CHOMP        => '-';
+use constant DIR_COLLAPSE     => '~';
+use constant DIR_CHOMP_NONE   => '^';
+
+# token related indexes
+use constant TOKEN_STR        =>  0;
+use constant TOKEN_ID         =>  1;
+use constant TOKEN_CHOMP      =>  2;
+use constant TOKEN_TRIGGER    =>  3;
+
+use constant TOKEN_CHOMP_NEXT =>  0; # sub-key for TOKEN_CHOMP
+use constant TOKEN_CHOMP_PREV =>  1; # sub-key for TOKEN_CHOMP
+
+use constant LAST_TOKEN       => -1;
+use constant PREVIOUS_TOKEN   => -2;
 
 # SHA seems to be more accurate, so we'll try them first.
 # Pure-Perl ones are slower, but they are fail-safes.
@@ -72,26 +106,6 @@ use constant DIGEST_MODS => qw(
    Digest::Perl::MD5
 );
 
-# see _parse();
-use constant MAP_KEYS_CHECK => sub {
-   my $tmp = q(
-         <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
-                  ? (
-                     defined <%HASH%>->{"<%KEY%>"}
-                     ? <%HASH%>->{"<%KEY%>"}
-                     : "[ERROR] Key not defined: <%KEY%>"
-                     )
-                  : "[ERROR] Invalid key: <%KEY%>"
-                  ;
-   );
-   $tmp =~ s/\n//xmsg;
-   $tmp =~ s/\s{2,}/ /xmsg;
-   return $tmp;
-}->();
-
-use constant MAP_KEYS_INIT     => q(<%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';);
-use constant MAP_KEYS_DEFAULT  => q(<%BUF%> .= <%HASH%>->{"<%KEY%>"};);
-
 use constant FRAGMENT_TMP      => <<'TEMPLATE_CONSTANT';
 
 # BEGIN TIDIED FRAGMENT
@@ -126,7 +140,9 @@ use constant DISK_CACHE_COMMENT => <<"TEMPLATE_CONSTANT";
 #
 TEMPLATE_CONSTANT
 
-use constant DISK_CACHE_MARKER => q{# This file is automatically generated by } . PARENT;
+use constant DISK_CACHE_MARKER => q{# This file is automatically generated by }
+                               .  PARENT
+                               ;
 
 use constant RESUME_TEMPLATE => sub {
    my $tmp = q~
@@ -200,9 +216,6 @@ BEGIN {
                         FRAGMENT_TMP
                         DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
-                        MAP_KEYS_CHECK
-                        MAP_KEYS_INIT
-                        MAP_KEYS_DEFAULT
                      )],
       delims    =>   [qw(
                         DELIM_START
@@ -215,24 +228,33 @@ BEGIN {
                         DELETE_WS
                         FAKER
                         FAKER_HASH
+                        FAKER_SELF
                         CACHE
                         CACHE_DIR
                         CACHE_OBJECT
+                        MONOLITH
                         IO_OBJECT
                         STRICT
                         SAFE
                         HEADER
                         ADD_ARGS
                         WARN_IDS
-                        FIX_UNCUDDLED
                         TYPE
+                        TYPE_FILE
                         COUNTER
+                        COUNTER_INCLUDE
+                        INSIDE_INCLUDE
+                        NEEDS_OBJECT
                         CID
                         FILENAME
                         RESUME
                         IOLAYER
                         STACK
                         USER_THANDLER
+                        DEEP_RECURSION
+                        INCLUDE_PATHS
+                        PRE_CHOMP
+                        POST_CHOMP
                         MAXOBJFIELD
                      )],
       resume    =>   [qw(
@@ -244,12 +266,43 @@ BEGIN {
                         RESUME_LOOP
                         RESUME_TEMPLATE
                      )],
+      chomp     =>   [qw(
+                        CHOMP_NONE
+                        COLLAPSE_NONE
+                        CHOMP_ALL
+                        CHOMP_LEFT
+                        CHOMP_RIGHT
+                        COLLAPSE_LEFT
+                        COLLAPSE_RIGHT
+                        COLLAPSE_ALL
+                     )],
+      directive =>   [qw(
+                        DIR_CHOMP
+                        DIR_COLLAPSE
+                        DIR_CHOMP_NONE
+                        DIR_CAPTURE
+                        DIR_DYNAMIC
+                        DIR_STATIC
+                        DIR_NOTADELIM
+                        DIR_COMMENT
+                     )],
+      token     =>   [qw(
+                        TOKEN_ID
+                        TOKEN_STR
+                        TOKEN_CHOMP
+                        TOKEN_TRIGGER
+                        TOKEN_CHOMP_NEXT
+                        TOKEN_CHOMP_PREV
+                        LAST_TOKEN
+                        PREVIOUS_TOKEN
+                      )],
       etc       =>   [qw(
                         DIGEST_MODS
                         STAT_MTIME
                         RE_DUMP_ERROR
                         STAT_SIZE
                         RE_NONFILE
+                        MAX_RECURSION
                      )],
    );
 
